#include <assert.h>
#include <stdio.h>
#include "frames.h"
#include "dspc.h"

#ifdef _DEBUG
#ifdef _WINDOWS
#ifdef _AFXDLL
	#define new DEBUG_NEW
#endif
#endif
#endif

Frames::Frames()
{
	Initialized = false;
	InputVector = 0;
	InputVector_pos = 0;

	// Space for one waveform frame
	Frame = new float [VECTOR_SIZE];
	Frame_backup = new float [VECTOR_SIZE];
	Frame_pos = 0;
}

Frames::~Frames()
{
	delete [] Frame;
	delete [] Frame_backup;
}

void Frames::addWaveform(float *waveform, int len)
{
	assert(waveform != 0);
	InputVector = waveform;
	InputVector_pos = 0;
	InputVector_len = len;
}

void Frames::processFrame(float *frame, float *ret_features)
{
	// Other parameterisation can be here
	int i;
	for(i = 0; i < VECTOR_SIZE; i++)
		ret_features[i] = frame[i];
}

int Frames::getFeatures(float *ret_features)
{
	if(InputVector == 0)
		return 0;                     // There is not input data

	// Copy one frame from the input buffer to the temporary one when the first pass through this function.
	// Shift the temporal buffer left an copy next 10 ms from the input buffer in all follow passes.
	if(!Initialized)
	{
		sCopy(VECTOR_SIZE, Frame, InputVector);
                                       // InputVector -> Frame

		// Backup the frame
		sCopy(VECTOR_SIZE, Frame_backup, Frame);
		Frame_pos = VECTOR_SIZE;
		InputVector_pos += VECTOR_SIZE;
		Initialized = true;
	}
	else
	{
		// restore the last frame and shift 10 ms left if the last frame has been processed
		if(Frame_pos == VECTOR_SIZE - STEP)
		{
			sCopy(VECTOR_SIZE, Frame, Frame_backup);  // Frame_backup -> Frame
			sShiftLeft(VECTOR_SIZE, Frame, STEP);
		}

		int n_input = InputVector_len - InputVector_pos;
		int n_output = VECTOR_SIZE - Frame_pos;
		int n_processed = (n_input < n_output ? n_input : n_output);
		sCopy(n_processed, Frame + Frame_pos, InputVector + InputVector_pos);
		InputVector_pos += n_processed;
		Frame_pos += n_processed;

		if(InputVector_pos > InputVector_len)
			InputVector = 0;
	}

	if(Frame_pos == VECTOR_SIZE)
	{
		// Backup the frame
		sCopy(VECTOR_SIZE, Frame_backup, Frame);

		Frame_pos = VECTOR_SIZE - STEP;

		// Perform parameterisation of one frame
		processFrame(Frame, ret_features);
		return 1;
	}

	return 0;
}

void Frames::reset()
{
	InputVector = (float *)0;
	Initialized = false;
}
